@@ -1,32 +1,29 @@
-/*
- *  apps/obexftp.c: Transfer from/to Mobile Equipment via OBEX
- *
- *  Copyright (c) 2002 Christian W. Zuckschwerdt <zany@triq.net>
- *
- *  This program is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by the Free
- *  Software Foundation; either version 2 of the License, or (at your option)
- *  any later version.
- *
- *  This program is distributed in the hope that it will be useful, but
- *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- *  for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *     
- */
-/*
- * Created at:    Don, 17 Jan 2002
+/**
+	\file apps/obexftp.c
+	Command line client to transfer from/to Mobile Equipment via OBEX.
+	ObexFTP library - language bindings for OBEX file transfer.
+
+	Copyright (c) 2002-2007 Christian W. Zuckschwerdt <zany@triq.net>
+
+	ObexFTP is free software; you can redistribute it and/or modify
+	it under the terms of the GNU Lesser General Public License as
+	published by the Free Software Foundation; either version 2 of
+	the License, or (at your option) any later version.
+
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU Lesser General Public License for more details.
+
+	You should have received a copy of the GNU Lesser General Public
+	License along with ObexFTP. If not, see <http://www.gnu.org/>.
  */
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#define _GNU_SOURCE
 #include <getopt.h>
 #include <errno.h>
 
@@ -36,18 +33,16 @@
 #include <obexftp/client.h>
 #include <obexftp/uuid.h>
 
-#ifdef _WIN32_FIXME
-/* OpenOBEX won't define a handler on win32 */
-void DEBUG(unsigned int n, ...) { }
-void DUMPBUFFER(unsigned int n, char *label, char *msg) { }
+#ifdef _WIN32
+#define strcasestr strstr
+#define ETIMEDOUT	WSAETIMEDOUT
+#define ECONNREFUSED	WSAECONNREFUSED
+#define EHOSTDOWN	WSAEHOSTDOWN
+#define EINPROGRESS	WSAEINPROGRESS
 #endif /* _WIN32 */
 
 #include <common.h>
 
-//#ifdef HAVE_BLUETOOTH
-#include "bt_discovery.h"
-//#endif
-
 // perhaps this scheme would be better?
 // IRDA		irda://[nick?]
 // CABLE	tty://path
@@ -106,7 +101,7 @@ static void info_cb(int event, const char *msg, int len, void *UNUSED(data))
 		break;
 
 	case OBEXFTP_EV_INFO:
-		printf("Got info %d: \n", (int)msg); // 64 bit problems ?
+		printf("Got info %u: \n", *(uint32_t*)msg);
 		break;
 
 	case OBEXFTP_EV_BODY:
@@ -131,9 +126,10 @@ static void info_cb(int event, const char *msg, int len, void *UNUSED(data))
 }
 
 /* create global uuid buffers */
-static const char *fbs_uuid = UUID_FBS;
-static const char *irmc_uuid = UUID_IRMC;
-static const char *s45_uuid = UUID_S45;
+static const char *fbs_uuid = (const char *)UUID_FBS;
+static const char *irmc_uuid = (const char *)UUID_IRMC;
+static const char *s45_uuid = (const char *)UUID_S45;
+static const char *pcsoftware_uuid = (const char *)UUID_PCSOFTWARE;
 
 /* parse UUID string to real bytes */
 static int parse_uuid(char *name, const char **uuid, int *uuid_len)
@@ -169,11 +165,69 @@ static int parse_uuid(char *name, const char **uuid, int *uuid_len)
 		if (uuid_len) *uuid_len = sizeof(UUID_S45);
 		return sizeof(UUID_S45);
 	}
+	
+        if (!strncasecmp(name, "pcsoftware", 10) || !strncasecmp(name, "sharp", 5)) {
+		fprintf(stderr, "Using PCSOFTWARE uuid.\n");
+		if (uuid) *uuid = pcsoftware_uuid;
+		if (uuid_len) *uuid_len = sizeof(UUID_PCSOFTWARE);
+		return sizeof(UUID_PCSOFTWARE);
+	}
 
 	return -1;
 }
 
+static void discover_usb()
+{
+	char **devices;
+	char **dev;
+	int interfaces_number;
+
+       	devices = obexftp_discover(OBEX_TRANS_USB);
+	interfaces_number = 0;
+	for(dev = devices; *dev; dev++) interfaces_number++;
+	printf("Found %d USB OBEX interfaces\n\n", interfaces_number);
+       	for(dev = devices; *dev; dev++) {
+		fprintf(stderr, "%s\n", *dev);
+       	}
+	printf("\nUse '-u interface_number' to connect\n");
+}
+
+static int find_bt(char *addr, char **res_bdaddr, int *res_channel)
+{
+	char **devices;
+	char **dev;
+
+	*res_bdaddr = addr;
+	if (!addr || strlen(addr) < (6*2+5) || addr[2]!=':') {
+  		fprintf(stderr, "Scanning for %s ...\n", addr);
+		devices = obexftp_discover(OBEX_TRANS_BLUETOOTH);
+  
+		for(dev = devices; dev && *dev; dev++) {
+      			if (!addr || strcasestr(*dev, addr)) {
+				fprintf(stderr, "Found: %s\n", *dev);
+				*res_bdaddr = *dev;
+				break;
+			}
+       			fprintf(stderr, "Seen: %s\n", *dev);
+		}
+	}
+	if (!*res_bdaddr)
+		return -1; /* No (matching) BT device found */
+//	(*res_bdaddr)[17] = '\0';
+  
+       	if (*res_channel < 0) {
+		fprintf(stderr, "Browsing %s ...\n", *res_bdaddr);
+		*res_channel = obexftp_browse_bt_ftp(*res_bdaddr);
+	}
+	if (*res_channel < 0)
+		return -1; /* No valid BT channel found */
+
+	return 0;
+}
+
+
 /*@only@*/ /*@null@*/ static obexftp_client_t *cli = NULL;
+static const char *src_dev = NULL;
 #ifdef HAVE_BLUETOOTH
 static int transport = OBEX_TRANS_BLUETOOTH;
 #else
@@ -181,7 +235,7 @@ static int transport = OBEX_TRANS_IRDA;
 #endif /* HAVE_BLUETOOTH */
 /*@only@*/ /*@null@*/ static char *device = NULL;
 static int channel = -1;
-static const char *use_uuid = UUID_FBS;
+static const char *use_uuid = (const char *)UUID_FBS;
 static int use_uuid_len = sizeof(UUID_FBS);
 static int use_conn=1;
 static int use_path=1;
@@ -190,7 +244,7 @@ static int use_path=1;
 /* connect with given uuid. re-connect every time */
 static int cli_connect_uuid(const char *uuid, int uuid_len)
 {
-	int retry;
+	int ret, retry;
 
 	if (cli == NULL) {
 
@@ -211,8 +265,32 @@ static int cli_connect_uuid(const char *uuid, int uuid_len)
 	for (retry = 0; retry < 3; retry++) {
 
 		/* Connect */
-		if (obexftp_connect_uuid (cli, device, channel, uuid, uuid_len) >= 0)
+		ret = obexftp_connect_src(cli, src_dev, device, channel, uuid, uuid_len);
+		if (ret >= 0)
        			return TRUE;
+		switch (errno) {
+		
+		case ETIMEDOUT:
+			perror("The device may have moved out of range");
+			break; /* retry */
+
+		case ECONNREFUSED:
+			perror("The user may have rejected the transfer");
+			return FALSE;
+
+		case EHOSTDOWN:
+			perror("The device may be out of range or turned off");
+			break; /* retry */
+
+		case EINPROGRESS:
+			perror("Interupted/bad reception or the device moved out of range");
+			break; /* retry */
+
+		default:
+			perror("error on connect()");
+			break;
+		} 
+
 		fprintf(stderr, "Still trying to connect\n");
 	}
 
@@ -228,6 +306,11 @@ static int cli_connect()
 		return TRUE;
 	}
 
+	/* complete bt address if necessary */
+	if (transport == OBEX_TRANS_BLUETOOTH) {
+		find_bt(device, &device, &channel);
+		// we should free() the find_bt result at some point
+	}
 	if (!cli_connect_uuid(use_uuid, use_uuid_len))
 		exit(1);
 
@@ -337,48 +420,12 @@ static void probe_device()
 }
 
 
-#ifdef HAVE_USB
-static void discover_cb(obex_t *handle, obex_object_t *object, int mode, int event, int obex_cmd, int obex_rsp)
-{
-	(void) handle;
-	(void) object;
-	(void) mode;
-	(void) event;
-	(void) obex_cmd;
-	(void) obex_rsp;
-}
-
-static void discover_usb()
-{
-	obex_t *handle;
-	obex_interface_t* obex_intf;
-	int i, interfaces_number;
-
-	if(! (handle = OBEX_Init(OBEX_TRANS_USB, discover_cb, 0))) {
-		printf( "OBEX_Init failed\n");
-		return;
-	}
-	interfaces_number = OBEX_FindInterfaces(handle, &obex_intf);
-	printf("Found %d USB OBEX interfaces\n", interfaces_number);
-	for (i=0; i < interfaces_number; i++)
-		printf("Interface %d:\n\tManufacturer: %s\n\tProduct: %s\n\tInterface description: %s\n", i,
-			obex_intf[i].usb.manufacturer,
-			obex_intf[i].usb.product,
-		       	obex_intf[i].usb.control_interface);
-	printf("Use '-u interface_number' to connect\n");
-	OBEX_Cleanup(handle);
-}
-#endif /* HAVE_USB */
-
-
 int main(int argc, char *argv[])
 {
 	int verbose=0;
 	int most_recent_cmd = 0;
-	char *target_path = NULL;
 	char *output_file = NULL;
 	char *move_src = NULL;
-	/* char *inbox; */
 
 	/* preset mode of operation depending on our name */
 	if (strstr(argv[0], "ls") != NULL)	most_recent_cmd = 'l';
@@ -392,26 +439,22 @@ int main(int argc, char *argv[])
 	}
 	if (channel >= 0) {
 		transport = OBEX_TRANS_USB;
-		fprintf(stderr, "Using USB: %d\n", channel);
+		fprintf(stderr, "Presetting to USB: %d\n", channel);
 	}
 	if (getenv(OBEXFTP_CABLE) != NULL) {
-		device = strdup(getenv(OBEXFTP_CABLE));
+		device = getenv(OBEXFTP_CABLE);
 		transport = OBEX_TRANS_CUSTOM;
-		fprintf(stderr, "Using TTY: %s\n", device);
+		fprintf(stderr, "Presetting to TTY: %s\n", device);
 	}
 	if (getenv(OBEXFTP_BLUETOOTH) != NULL) {
 		device = getenv(OBEXFTP_BLUETOOTH);
-		if (channel <= 0 || strlen(device) < (6*2+5) || device[2]!=':')
-			discover_bt(device, &device, &channel);
-		else
-			device = strdup(device);
 		transport = OBEX_TRANS_BLUETOOTH;
-		fprintf(stderr, "Using BT: %s (%d)\n", device, channel);
+		fprintf(stderr, "Presetting to BT: %s (%d)\n", device, channel);
 	}
 	if (getenv(OBEXFTP_INET) != NULL) {
-		device = strdup(getenv(OBEXFTP_INET));
+		device = getenv(OBEXFTP_INET);
 		transport = OBEX_TRANS_INET;
-		fprintf(stderr, "Using INET: %s\n", device);
+		fprintf(stderr, "Presetting to INET: %s\n", device);
 	}
 	       
 
@@ -421,8 +464,9 @@ int main(int argc, char *argv[])
 			{"irda",	no_argument, NULL, 'i'},
 #ifdef HAVE_BLUETOOTH
 			{"bluetooth",	optional_argument, NULL, 'b'},
-#endif
 			{"channel",	required_argument, NULL, 'B'},
+			{"hci",		required_argument, NULL, 'd'},
+#endif
 #ifdef HAVE_USB
 			{"usb",		optional_argument, NULL, 'u'},
 #endif
@@ -434,7 +478,6 @@ int main(int argc, char *argv[])
 			{"list",	optional_argument, NULL, 'l'},
 			{"chdir",	required_argument, NULL, 'c'},
 			{"mkdir",	required_argument, NULL, 'C'},
-			{"path",	required_argument, NULL, 'f'},
 			{"output",	required_argument, NULL, 'o'},
 			{"get",		required_argument, NULL, 'g'},
 			{"getdelete",	required_argument, NULL, 'G'},
@@ -451,7 +494,7 @@ int main(int argc, char *argv[])
 			{0, 0, 0, 0}
 		};
 		
-		c = getopt_long (argc, argv, "-ib::B:u::t:n:U::HSL::l::c:C:f:o:g:G:p:k:XYxm:VvhN:FP",
+		c = getopt_long (argc, argv, "-ib::B:d:u::t:n:U::HSL::l::c:C:f:o:g:G:p:k:XYxm:VvhN:FP",
 				 long_options, &option_index);
 		if (c == -1)
 			break;
@@ -463,8 +506,6 @@ int main(int argc, char *argv[])
 		
 		case 'i':
 			transport = OBEX_TRANS_IRDA;
-			if (device != NULL)
-				free(device);
        			device = NULL;
 			channel = 0;
 			break;
@@ -472,30 +513,30 @@ int main(int argc, char *argv[])
 #ifdef HAVE_BLUETOOTH
 		case 'b':
 			transport = OBEX_TRANS_BLUETOOTH;
-			if (device != NULL)
-				free(device);
-       			//device = optarg;
 			/* handle severed optional option argument */
 			if (!optarg && argc > optind && argv[optind][0] != '-') {
 				optarg = argv[optind];
 				optind++;
 			}
-			discover_bt(optarg, &device, &channel);
-			//fprintf(stderr, "Got %s channel %d\n", device, channel);
+       			device = optarg;
 			break;
 			
 		case 'B':
 			channel = atoi(optarg);
 			break;
+			
+		case 'd':
+			src_dev = optarg;
+			break;
 #endif /* HAVE_BLUETOOTH */
 
 #ifdef HAVE_USB
 		case 'u':
+#ifndef _WIN32
 			if (geteuid() != 0)
 				fprintf(stderr, "If USB doesn't work setup permissions in udev or run as superuser.\n");
+#endif
 			transport = OBEX_TRANS_USB;
-			if (device != NULL)
-				free(device);
        			device = NULL;
 			/* handle severed optional option argument */
 			if (!optarg && argc > optind && argv[optind][0] != '-') {
@@ -510,9 +551,7 @@ int main(int argc, char *argv[])
 		
 		case 't':
 			transport = OBEX_TRANS_CUSTOM;
-			if (device != NULL)
-				free(device); /* ok to to free an optarg? */
-       			device = optarg; /* strdup? */
+       			device = optarg;
 			channel = 0;
 
 			if (strstr(optarg, "ir") != NULL)
@@ -523,9 +562,7 @@ int main(int argc, char *argv[])
 			fprintf(stderr,"Option -%c is deprecated, use -%c instead\n",'N','n');
 		case 'n':
 			transport = OBEX_TRANS_INET;
-			if (device != NULL)
-				free(device); /* ok to to free an optarg? */
-       			device = optarg; /* strdup? */
+       			device = optarg;
 			channel = 0;
 
 			{
@@ -570,7 +607,8 @@ int main(int argc, char *argv[])
 					stat_entry_t *st;
 					st = obexftp_stat(cli, ent->name);
 					if (!st) continue;
-					printf("%d %s (%d)\n", st->size, ent->name, ent->mode);
+					printf("%d %s%s\n", st->size, ent->name,
+						ent->mode&S_IFDIR?"/":"");
 				}
 				obexftp_closedir(dir);
 			}
@@ -606,10 +644,6 @@ int main(int argc, char *argv[])
 			most_recent_cmd = c;
 			break;
 
-		case 'f':
-			target_path = optarg;
-			break;
-
 		case 'o':
 			output_file = optarg;
 			break;
@@ -713,19 +747,20 @@ int main(int argc, char *argv[])
 #ifdef HAVE_BLUETOOTH
 				" -b, --bluetooth [<device>]  use or search a bluetooth device\n"
 				" [ -B, --channel <number> ]  use this bluetooth channel when connecting\n"
+				" [ -d, --hci <no/address> ]  use source device with this address or number\n"
 #endif
 #ifdef HAVE_USB
 				" -u, --usb [<intf>]          connect to a usb interface or list interfaces\n"
 #endif
 				" -t, --tty <device>          connect to this tty using a custom transport\n"
 				" -n, --network <host>        connect to this host\n\n"
-				" -U, --uuid                  use given uuid (none, FBS, IRMC, S45)\n"
+				" -U, --uuid                  use given uuid (none, FBS, IRMC, S45, SHARP)\n"
 				" -H, --noconn                suppress connection ids (no conn header)\n"
 				" -S, --nopath                dont use setpaths (use path as filename)\n\n"
 				" -c, --chdir <DIR>           chdir\n"
 				" -C, --mkdir <DIR>           mkdir and chdir\n"
 				" -l, --list [<FOLDER>]       list current/given folder\n"
-				// " -f, --path <PATH>           specify the local file name or directory\n"
+				" -o, --output <PATH>         specify the target file name\n"
 				"                             get and put always specify the remote name.\n"
 				" -g, --get <SOURCE>          fetch files\n"
 				" -G, --getdelete <SOURCE>    fetch and delete (move) files \n"
