@@ -1,31 +1,33 @@
-/*
- *  obexftp/client.c: ObexFTP client library
- *
- *  Copyright (c) 2002 Christian W. Zuckschwerdt <zany@triq.net>
- *
- *  This program is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by the Free
- *  Software Foundation; either version 2 of the License, or (at your option)
- *  any later version.
- *
- *  This program is distributed in the hope that it will be useful, but
- *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- *  for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *     
+/**
+	\file obexftp/client.c
+	ObexFTP client API implementation.
+	ObexFTP library - language bindings for OBEX file transfer.
+
+	Copyright (c) 2002-2007 Christian W. Zuckschwerdt <zany@triq.net>
+
+	ObexFTP is free software; you can redistribute it and/or modify
+	it under the terms of the GNU Lesser General Public License as
+	published by the Free Software Foundation; either version 2 of
+	the License, or (at your option) any later version.
+
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU Lesser General Public License for more details.
+
+	You should have received a copy of the GNU Lesser General Public
+	License along with ObexFTP. If not, see <http://www.gnu.org/>.
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <string.h>
+#define _GNU_SOURCE
+#include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <string.h>
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <errno.h>
@@ -37,22 +39,19 @@
 #include <time.h>
 
 #ifdef _WIN32
-#include <winsock.h>
+#define O_BINARY (_O_BINARY)
+#define CREATE_MODE_FILE (S_IRUSR|S_IWUSR)
 #else
 #include <sys/socket.h>
 #include <arpa/inet.h>
 #include <netinet/in.h>
+#define O_BINARY (0)
+#define CREATE_MODE_FILE (S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH)
 #endif /* _WIN32 */
 
 #ifdef HAVE_BLUETOOTH
-#ifdef __FreeBSD__
-#include <sys/types.h>
-#include <bluetooth.h>
-#else /* Linux */
-#include <bluetooth/bluetooth.h>
-#include <bluetooth/rfcomm.h>
-#endif /* __FreeBSD__ */
-#endif
+#include "bt_kit.h"
+#endif /* HAVE_BLUETOOTH */
 
 #include <openobex/obex.h>
 #include <multicobex/multi_cobex.h>
@@ -67,23 +66,29 @@
 #include <common.h>
 
 
+#pragma pack(1)
 typedef struct { /* fixed to 6 bytes for now */
 	uint8_t code;
 	uint8_t info_len;
 	uint8_t info[4];
 } apparam_t;
+#pragma pack()
 
 
+/**
+	Empty callback used as default.
+ */
 static void dummy_info_cb(int UNUSED(event), const char *UNUSED(msg), int UNUSED(len), void *UNUSED(data))
 {
-	/* dummy */
+	/* do nothing */
 }
 
 
-/*
- * Normalize the path argument
- * wont turn relative paths into (most likely wrong) absolute ones
- * wont expand "../" or "./"
+/**
+	Normalize the path argument.
+	\note
+	wont turn relative paths into (most likely wrong) absolute ones.
+	wont expand "../" or "./".
  */
 /*
 static char *normalize_file_path(const char *name)
@@ -113,12 +118,15 @@ static char *normalize_file_path(const char *name)
 }
 */
 
-/*
- * Normalize the path argument and split into pathname and basename
- * wont turn relative paths into (most likely wrong) absolute ones
- * wont expand "../" or "./"
- * Will keep "telecom" prefix
- * Do not use this function if there is no slash in the argument!
+
+/**
+	Normalize the path argument and split into pathname and basename.
+	\note
+	Wont turn relative paths into (most likely wrong) absolute ones.
+	Wont expand "../" or "./".
+	Will keep "telecom" prefix.
+	\warning
+	Do not use this function if there is no slash in the argument!
  */
 static void split_file_path(const char *name, /*@only@*/ char **basepath, /*@only@*/ char **basename)
 {
@@ -170,9 +178,12 @@ static void split_file_path(const char *name, /*@only@*/ char **basepath, /*@onl
 }
 
 
-/* Add more data from memory to stream. */
+/**
+	Add more data from memory to stream.
+ */
 static int cli_fillstream_from_memory(obexftp_client_t *cli, obex_object_t *object)
 {
+	obex_headerdata_t hv;
 	int actual = cli->out_size - cli->out_pos;
 	if (actual > STREAM_CHUNK)
 		actual = STREAM_CHUNK;
@@ -180,29 +191,36 @@ static int cli_fillstream_from_memory(obexftp_client_t *cli, obex_object_t *obje
 	
 	if(actual > 0) {
 		/* Read was ok! */
+		hv.bs = (const uint8_t *) &cli->out_data[cli->out_pos];
 		(void) OBEX_ObjectAddHeader(cli->obexhandle, object, OBEX_HDR_BODY,
-				(obex_headerdata_t) (const uint8_t *) &cli->out_data[cli->out_pos], actual, OBEX_FL_STREAM_DATA);
+				hv, actual, OBEX_FL_STREAM_DATA);
 		cli->out_pos += actual;
 	}
 	else if(actual == 0) {
 		/* EOF */
 		cli->out_data = NULL; /* dont free, isnt ours */
+		hv.bs = (const uint8_t *) &cli->out_data[cli->out_pos];
 		(void) OBEX_ObjectAddHeader(cli->obexhandle, object, OBEX_HDR_BODY,
-				(obex_headerdata_t) (const uint8_t *) &cli->out_data[cli->out_pos], 0, OBEX_FL_STREAM_DATAEND);
+				hv, 0, OBEX_FL_STREAM_DATAEND);
 	}
 	else {
 		/* Error */
 		cli->out_data = NULL; /* dont free, isnt ours */
+		hv.bs = (const uint8_t *) NULL;
 		(void) OBEX_ObjectAddHeader(cli->obexhandle, object, OBEX_HDR_BODY,
-				(obex_headerdata_t) (const uint8_t *) NULL, 0, OBEX_FL_STREAM_DATA);
+				hv, 0, OBEX_FL_STREAM_DATA);
 	}
 
 	return actual;
 }
 
-/* Add more data from file to stream. */
+
+/**
+	Add more data from file to stream.
+ */
 static int cli_fillstream_from_file(obexftp_client_t *cli, obex_object_t *object)
 {
+	obex_headerdata_t hv;
 	int actual;
 		
 	DEBUG(3, "%s()\n", __func__);
@@ -213,30 +231,34 @@ static int cli_fillstream_from_file(obexftp_client_t *cli, obex_object_t *object
 	
 	if(actual > 0) {
 		/* Read was ok! */
+		hv.bs = (const uint8_t *) cli->stream_chunk;
 		(void) OBEX_ObjectAddHeader(cli->obexhandle, object, OBEX_HDR_BODY,
-				(obex_headerdata_t) (const uint8_t *) cli->stream_chunk, actual, OBEX_FL_STREAM_DATA);
+				hv, actual, OBEX_FL_STREAM_DATA);
 	}
 	else if(actual == 0) {
 		/* EOF */
 		(void) close(cli->fd);
 		cli->fd = -1;
+		hv.bs = (const uint8_t *) cli->stream_chunk;
 		(void) OBEX_ObjectAddHeader(cli->obexhandle, object, OBEX_HDR_BODY,
-				(obex_headerdata_t) (const uint8_t *) cli->stream_chunk, 0, OBEX_FL_STREAM_DATAEND);
+				hv, 0, OBEX_FL_STREAM_DATAEND);
 	}
         else {
 		/* Error */
 		(void) close(cli->fd);
 		cli->fd = -1;
+		hv.bs = (const uint8_t *) NULL;
 		(void) OBEX_ObjectAddHeader(cli->obexhandle, object, OBEX_HDR_BODY,
-				(obex_headerdata_t) (const uint8_t *) NULL,
-				0, OBEX_FL_STREAM_DATA);
+				hv, 0, OBEX_FL_STREAM_DATA);
 	}
 
 	return actual;
 }
 
 
-/* Save body from object or return application parameters */
+/**
+	Save body from object or return application parameters.
+ */
 static void client_done(obex_t *handle, obex_object_t *object, int UNUSED(obex_cmd), int UNUSED(obex_rsp))
 {
 	obex_headerdata_t hv;
@@ -298,10 +320,10 @@ static void client_done(obex_t *handle, obex_object_t *object, int UNUSED(obex_c
 			DEBUG(3, "%s() Found application parameters\n", __func__);
                         if(hlen == sizeof(apparam_t)) {
 				app = (const apparam_t *)hv.bs;
-				 /* needed for alignment */
-				memcpy(&cli->apparam_info, &app->info, sizeof(cli->apparam_info));
-				cli->apparam_info = ntohl(cli->apparam_info); // 64 bit problems?
-				cli->infocb(OBEXFTP_EV_INFO, (void *)cli->apparam_info, 0, cli->infocb_data);
+				/* order is network byte order (big-endian) */
+				cli->apparam_info = (app->info[0] << (3*8)) + (app->info[1] << (2*8)) +
+				                    (app->info[2] << (1*8)) + (app->info[3] << (0*8));
+				cli->infocb(OBEXFTP_EV_INFO, (char*)&cli->apparam_info, 0, cli->infocb_data);
 			}
 			else
 				DEBUG(3, "%s() Application parameters don't fit %d vs. %d.\n", __func__, hlen, sizeof(apparam_t));
@@ -318,8 +340,7 @@ static void client_done(obex_t *handle, obex_object_t *object, int UNUSED(obex_c
 				/* simple body writer */
 				int fd;
 				//fd = open_safe("", cli-> target_fn);
-				fd = creat(cli-> target_fn,
-					   S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
+				fd = open(cli-> target_fn, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, CREATE_MODE_FILE);
 				if(fd > 0) {
 					(void) write(fd, body_data, body_len);
 					(void) close(fd);
@@ -343,7 +364,9 @@ static void client_done(obex_t *handle, obex_object_t *object, int UNUSED(obex_c
 }
 
 
-/* Incoming event from OpenOBEX. */
+/**
+	Handle incoming event from OpenOBEX.
+ */
 static void cli_obex_event(obex_t *handle, obex_object_t *object, int UNUSED(mode), int event, int obex_cmd, int obex_rsp)
 {
 	/*@temp@*/ obexftp_client_t *cli;
@@ -385,7 +408,9 @@ static void cli_obex_event(obex_t *handle, obex_object_t *object, int UNUSED(mod
 }
 
 
-/* Do an OBEX request sync. */
+/**
+	Wait for the OBEX client to finish.
+ */
 static int obexftp_sync(obexftp_client_t *cli)
 {
 	int ret;
@@ -408,7 +433,11 @@ static int obexftp_sync(obexftp_client_t *cli)
 	else
 		return -1;
 }
-	
+
+
+/**
+	Do an OBEX request synchronous.
+ */
 static int cli_sync_request(obexftp_client_t *cli, obex_object_t *object)
 {
 	DEBUG(3, "%s()\n", __func__);
@@ -420,9 +449,18 @@ static int cli_sync_request(obexftp_client_t *cli, obex_object_t *object)
 
 	return obexftp_sync (cli);
 }
-	
 
-/* Create an obexftp client */
+
+/**
+	Create an obexftp client.
+
+	\param transport the transport type that will be used
+	\param ctrans optional custom transport (don't use)
+	\param infocb optional info callback
+	\param infocb_data optional info callback data
+
+	\return a new allocated ObexFTP client instance, NULL on error
+ */
 obexftp_client_t *obexftp_open(int transport, /*const*/ obex_ctrans_t *ctrans, obexftp_info_cb_t infocb, void *infocb_data)
 {
 	obexftp_client_t *cli;
@@ -473,38 +511,81 @@ obexftp_client_t *obexftp_open(int transport, /*const*/ obex_ctrans_t *ctrans, o
 	}
 	return cli;
 }
-	
-/**
- * obexftp_close - Close an obexftp client and free the resources
- * @cli: the obexftp_client_t to be shut done and free'd.
- *
- * Close an obexftp client and free the resources.
- */
 
-/* Close an obexftp client and free the resources */
+
+/**
+	Close an obexftp client and free the resources.
+
+	\param cli
+		the obexftp_client_t to be shut done and free'd.
+		It's save to pass NULL here.
+
+	Closes the given obexftp client and frees the resources.
+	It's recommended to set the client reference to NULL afterwards.
+ */
 void obexftp_close(obexftp_client_t *cli)
 {
 	DEBUG(3, "%s()\n", __func__);
 	return_if_fail(cli != NULL);
 
 	OBEX_Cleanup(cli->obexhandle);
+	if (cli->buf_data) {
+		DEBUG(1, "%s: Warning: purging left-over buffer.\n", __func__);
+		free(cli->buf_data);
+	}
+	cache_purge(&cli->cache, NULL);
 	free(cli->stream_chunk);
 	free(cli);
 }
 
-/* Do connect as client */
+
+/**
+	Do simple connect as client.
+
+	\param cli an obexftp_client_t created by obexftp_open().
+	\param device the device address to connect to (transport specific)
+	\param port the port/channel for the device address
+	\param uuid UUID string for CONNECT (no default)
+	\param uuid_len length of the UUID string (excluding terminating zero)
+
+	\return the result of the CONNECT request, -1 on error
+
+	\note Wrapper function for obexftp_connect_src()
+	\warning Always use a UUID (except for OBEX PUSH)
+ */
 int obexftp_connect_uuid(obexftp_client_t *cli, const char *device, int port, const uint8_t uuid[], uint32_t uuid_len)
+{
+	return obexftp_connect_src(cli, NULL, device, port, uuid, uuid_len);
+}
+
+
+/**
+	Connect this ObexFTP client using a given source address by sending an OBEX CONNECT request.
+
+	\param cli an obexftp_client_t created by obexftp_open().
+	\param src optional local source interface address (transport specific)
+	\param device the device address to connect to (transport specific)
+	\param port the port/channel for the device address
+	\param uuid UUID string for CONNECT (no default)
+	\param uuid_len length of the UUID string (excluding terminating zero)
+
+	\return the result of the CONNECT request, -1 on error
+
+	\note Always use a UUID (except for OBEX PUSH)
+ */
+int obexftp_connect_src(obexftp_client_t *cli, const char *src, const char *device, int port, const uint8_t uuid[], uint32_t uuid_len)
 {
 	struct sockaddr_in peer;
 #ifdef HAVE_BLUETOOTH
 	char *devicedup, *devicep;
-	bdaddr_t bdaddr;
+	bdaddr_t bdaddr, src_addr;
 #endif
 #ifdef HAVE_USB
 	int obex_intf_cnt;
 	obex_interface_t *obex_intf;
 #endif
 	obex_object_t *object;
+	obex_headerdata_t hv;
 	int ret = -1; /* no connection yet */
 
 	DEBUG(3, "%s()\n", __func__);
@@ -524,9 +605,16 @@ int obexftp_connect_uuid(obexftp_client_t *cli, const char *device, int port, co
 			ret = -EINVAL;
 			break;
 		}
-		if (inet_aton(device, &peer.sin_addr)) {
+#ifdef _WIN32
+		peer.sin_addr.s_addr = inet_addr(device);
+		ret = (peer.sin_addr.s_addr == INADDR_NONE) ? 0 : 1;
+#else
+		ret = inet_aton(device, &peer.sin_addr);
+#endif
+		if (ret) {
+			peer.sin_family = AF_INET;
 			peer.sin_port = htons(port); /* overridden with OBEX_PORT 650 anyhow */
-			ret = OBEX_TransportConnect(cli->obexhandle, (struct sockaddr *) &peer,
+			ret = InOBEX_TransportConnect(cli->obexhandle, (struct sockaddr *) &peer,
 						  sizeof(struct sockaddr_in));
 			DEBUG(3, "%s() TCP %d\n", __func__, ret);
 		} else
@@ -547,12 +635,23 @@ int obexftp_connect_uuid(obexftp_client_t *cli, const char *device, int port, co
 
 #ifdef HAVE_BLUETOOTH
 	case OBEX_TRANS_BLUETOOTH:
+		if (!src) {
+			bacpy(&src_addr, BDADDR_ANY);
+		}
+#ifdef HAVE_SDPLIB
+		else if (!strncmp(src, "hci", 3)) {
+			hci_devba(atoi(src + 3), &src_addr);
+		}
+#endif
+		else {
+			str2ba(src, &src_addr);
+		}
 		if (!device) {
 			ret = -EINVAL;
 			break;
 		}
 		if (port < 1) {
-			port = obexftp_scan_bt(device, 0);
+			port = obexftp_browse_bt(device, 0);
 		}
 		/* transform some chars to colons */
 		devicedup = devicep = strdup(device);
@@ -561,9 +660,9 @@ int obexftp_connect_uuid(obexftp_client_t *cli, const char *device, int port, co
 			if (*devicep == '_') *devicep = ':';
 			if (*devicep == '/') *devicep = ':';
 		}
-		(void) str2ba(devicedup, &bdaddr); /* what is the meaning of the return code? */
+		(void) str2ba(devicedup, &bdaddr);
 		free(devicedup);
-		ret = BtOBEX_TransportConnect(cli->obexhandle, BDADDR_ANY, &bdaddr, (uint8_t)port);
+		ret = BtOBEX_TransportConnect(cli->obexhandle, &src_addr, &bdaddr, (uint8_t)port);
 		DEBUG(3, "%s() BT %d\n", __func__, ret);
 		break;
 #endif /* HAVE_BLUETOOTH */
@@ -599,10 +698,9 @@ int obexftp_connect_uuid(obexftp_client_t *cli, const char *device, int port, co
 #ifdef COMPAT_S45
 	// try S45 UUID first.
 	object = OBEX_ObjectNew(cli->obexhandle, OBEX_CMD_CONNECT);
+	hv.bs = UUID_S45;
 	if(OBEX_ObjectAddHeader(cli->obexhandle, object, OBEX_HDR_TARGET,
-                                (obex_headerdata_t) UUID_S45,
-                                sizeof(UUID_S45),
-                                OBEX_FL_FIT_ONE_PACKET) < 0)    {
+                                hv, sizeof(UUID_S45), OBEX_FL_FIT_ONE_PACKET) < 0) {
                 DEBUG(1, "Error adding header\n");
                 OBEX_ObjectDelete(cli->obexhandle, object);
                 return -1;
@@ -615,10 +713,9 @@ int obexftp_connect_uuid(obexftp_client_t *cli, const char *device, int port, co
 #endif
 		object = OBEX_ObjectNew(cli->obexhandle, OBEX_CMD_CONNECT);
 		if (uuid) {
+			hv.bs = uuid;
 			if(OBEX_ObjectAddHeader(cli->obexhandle, object, OBEX_HDR_TARGET,
-        	        	                (obex_headerdata_t) uuid,
-        		                        uuid_len,
-	                	                OBEX_FL_FIT_ONE_PACKET) < 0)    {
+        	        	                hv, uuid_len, OBEX_FL_FIT_ONE_PACKET) < 0) {
 	        	        DEBUG(1, "Error adding header\n");
 		                OBEX_ObjectDelete(cli->obexhandle, object);
                 		return -1;
@@ -633,17 +730,25 @@ int obexftp_connect_uuid(obexftp_client_t *cli, const char *device, int port, co
 #endif
 
 	if(ret < 0)
-		cli->infocb(OBEXFTP_EV_ERR, "FBS UUID", 0, cli->infocb_data);
+		cli->infocb(OBEXFTP_EV_ERR, "send UUID", 0, cli->infocb_data);
 	else
 		cli->infocb(OBEXFTP_EV_OK, "", 0, cli->infocb_data);
 
 	return ret;
 }
 
-/* Do disconnect as client */
+
+/**
+	Disconnect this ObexFTP client by sending an OBEX DISCONNECT request.
+
+	\param cli an obexftp_client_t created by obexftp_open().
+
+	\return the result of the DISCONNECT request
+ */
 int obexftp_disconnect(obexftp_client_t *cli)
 {
 	obex_object_t *object;
+	obex_headerdata_t hv;
 	int ret;
 
 	DEBUG(3, "%s()\n", __func__);
@@ -652,7 +757,11 @@ int obexftp_disconnect(obexftp_client_t *cli)
 	cli->infocb(OBEXFTP_EV_DISCONNECTING, "", 0, cli->infocb_data);
 
 	object = OBEX_ObjectNew(cli->obexhandle, OBEX_CMD_DISCONNECT);
-	(void) OBEX_ObjectAddHeader(cli->obexhandle, object, OBEX_HDR_CONNECTION, (obex_headerdata_t) cli->connection_id, sizeof(uint32_t), OBEX_FL_FIT_ONE_PACKET);
+	if(cli->connection_id != 0xffffffff) {
+		hv.bq4 = cli->connection_id;
+		(void) OBEX_ObjectAddHeader(cli->obexhandle, object, OBEX_HDR_CONNECTION,
+				    hv, sizeof(uint32_t), OBEX_FL_FIT_ONE_PACKET);
+	}
 	ret = cli_sync_request(cli, object);
 
 	if(ret < 0)
@@ -665,7 +774,16 @@ int obexftp_disconnect(obexftp_client_t *cli)
 	return ret;
 }
 
-/* Do an OBEX GET with app info opcode. */
+
+/**
+	Send a custom Siemens OBEX app info opcode.
+
+	\param cli an obexftp_client_t created by obexftp_open().
+	\param opcode the info opcode,
+		0x01 to inquire installed memory, 0x02 to get free memory
+
+	\return the result of the app info request
+ */
 int obexftp_info(obexftp_client_t *cli, uint8_t opcode)
 {
 	obex_object_t *object = NULL;
@@ -691,8 +809,20 @@ int obexftp_info(obexftp_client_t *cli, uint8_t opcode)
 	return ret;
 }
 
-/* Do an OBEX GET with optional TYPE. localname and remotename may be null. */
-/* Directories will be changed into first if split path quirk is set. */
+
+/**
+	Send an OBEX GET with optional TYPE.
+	Directories will be changed into first if split path quirk is set.
+
+	\param cli an obexftp_client_t created by obexftp_open().
+	\param type OBEX TYPE of the request
+	\param localname optional file to write
+	\param remotename OBEX NAME to request
+
+	\return the result of GET request
+
+	\note \a localname and \a remotename may be null.
+ */
 int obexftp_get_type(obexftp_client_t *cli, const char *type, const char *localname, const char *remotename)
 {
 	obex_object_t *object = NULL;
@@ -744,7 +874,15 @@ int obexftp_get_type(obexftp_client_t *cli, const char *type, const char *localn
 }
 
 
-/* Do an OBEX rename. */
+/**
+	Send an custom Siemens OBEX rename request.
+
+	\param cli an obexftp_client_t created by obexftp_open().
+	\param sourcename remote filename to be renamed
+	\param targetname remote target filename
+
+	\return the result of Siemens rename request
+ */
 int obexftp_rename(obexftp_client_t *cli, const char *sourcename, const char *targetname)
 {
 	obex_object_t *object = NULL;
@@ -772,7 +910,14 @@ int obexftp_rename(obexftp_client_t *cli, const char *sourcename, const char *ta
 }
 
 
-/* Do an OBEX PUT with empty file (delete) */
+/**
+	Send an OBEX PUT with empty file name (delete).
+
+	\param cli an obexftp_client_t created by obexftp_open().
+	\param name the remote filename/foldername to be removed. 
+
+	\return the result of the empty OBEX PUT request
+ */
 int obexftp_del(obexftp_client_t *cli, const char *name)
 {
 	obex_object_t *object;
@@ -816,7 +961,17 @@ int obexftp_del(obexftp_client_t *cli, const char *name)
 }
 
 
-/* Do OBEX SetPath -- handles NULL, "", "/" and everything else correctly */
+/**
+	Send OBEX SETPATH request (multiple requests if split path flag is set).
+
+	\param cli an obexftp_client_t created by obexftp_open().
+	\param name path to change into
+	\param create flag whether to create missing folders or fail
+
+	\return the result of the OBEX SETPATH request(s).
+
+	\note handles NULL, "", "/" and everything else correctly.
+ */
 int obexftp_setpath(obexftp_client_t *cli, const char *name, int create)
 {
 	obex_object_t *object;
@@ -837,9 +992,15 @@ int obexftp_setpath(obexftp_client_t *cli, const char *name, int create)
 			}
 	
 			cli->infocb(OBEXFTP_EV_SENDING, tail, 0, cli->infocb_data);
-			DEBUG(2, "%s() Setpath \"%s\"\n", __func__, tail);
-			object = obexftp_build_setpath (cli->obexhandle, cli->connection_id, tail, create);
+			DEBUG(2, "%s() Setpath \"%s\" (create:%d)\n", __func__, tail, create);
+			/* try without the create flag */
+			object = obexftp_build_setpath (cli->obexhandle, cli->connection_id, tail, 0);
 			ret = cli_sync_request(cli, object);
+			if ((ret < 0) && create) {
+				/* try again with create flag set maybe? */
+				object = obexftp_build_setpath (cli->obexhandle, cli->connection_id, tail, 1);
+				ret = cli_sync_request(cli, object);
+			}
 			if (ret < 0) break;
 
 			tail = p;
@@ -867,8 +1028,18 @@ int obexftp_setpath(obexftp_client_t *cli, const char *name, int create)
 	return ret;
 }
 
-/* Do an OBEX PUT, optionally with (some) setpath. */
-/* put to filename's basename if remotename is NULL or ends with a slash*/
+
+/**
+	Send an OBEX PUT, optionally with (some) SETPATHs for a local file.
+
+	\param cli an obexftp_client_t created by obexftp_open().
+	\param filename local file to send
+	\param remotename remote name to write
+
+	\return the result of the OBEX PUT (and SETPATH) request(s).
+
+	\note Puts to filename's basename if remotename is NULL or ends with a slash.
+ */
 int obexftp_put_file(obexftp_client_t *cli, const char *filename, const char *remotename)
 {
 	obex_object_t *object;
@@ -909,7 +1080,7 @@ int obexftp_put_file(obexftp_client_t *cli, const char *filename, const char *re
 		object = build_object_from_file (cli->obexhandle, cli->connection_id, filename, remotename);
 	}
 	
-	cli->fd = open(filename, O_RDONLY, 0);
+	cli->fd = open(filename, O_RDONLY | O_BINARY, 0);
 	if(cli->fd < 0)
 		ret = -1;
 	else {
@@ -928,7 +1099,19 @@ int obexftp_put_file(obexftp_client_t *cli, const char *filename, const char *re
 	return ret;
 }
 
-/* Put memory data - a remotename must be given always! */
+
+/**
+	Send memory data by OBEX PUT, optionally with (some) SETPATHs.
+
+	\param cli an obexftp_client_t created by obexftp_open().
+	\param data data to send
+	\param size length of the data
+	\param remotename remote name to write
+
+	\return the result of the OBEX PUT (and SETPATH) request(s).
+
+	\note A remotename must be given always.
+ */
 int obexftp_put_data(obexftp_client_t *cli, const char *data, int size,
 		     const char *remotename)
 {
@@ -978,8 +1161,10 @@ int obexftp_put_data(obexftp_client_t *cli, const char *data, int size,
 	return ret;
 }
 
-/* simple device discovery wrappers. USB and BT only */
 
+/**
+	Simple device discovery wrappers. USB and BT only.
+ */
 static char **discover_usb()
 {
 	char **res = NULL;
@@ -998,7 +1183,7 @@ static char **discover_usb()
 	
 	for (i=0; i < interfaces_number; i++) {
 		res[i] = malloc(201);
-		snprintf(res[i], 200, "USB:%d (Manufacturer: %s Product: %s Serial: %s Interface description: %s)", i,
+		snprintf(res[i], 200, "%d (Manufacturer: %s Product: %s Serial: %s Interface description: %s)", i,
 			obex_intf[i].usb.manufacturer,
 			obex_intf[i].usb.product,
 			obex_intf[i].usb.serial,
@@ -1010,6 +1195,66 @@ static char **discover_usb()
 	return res;
 }
 
+
+char **obexftp_discover_bt_src(const char *src)
+{
+#ifdef HAVE_BLUETOOTH
+	return btkit_discover(src);
+#else
+	return NULL;
+#endif /* HAVE_BLUETOOTH */
+}
+
+
+char *obexftp_bt_name_src(const char *addr, const char *src)
+{
+#ifdef HAVE_BLUETOOTH
+	return btkit_getname(src, addr);
+#else
+	return NULL;
+#endif /* HAVE_BLUETOOTH */
+}
+
+
+int obexftp_browse_bt_src(const char *src, const char *addr, int svclass)
+{
+#ifdef HAVE_BLUETOOTH
+	return btkit_browse(src, addr, svclass);
+#else
+	return 0;
+#endif /* HAVE_BLUETOOTH */
+}
+
+
+int obexftp_sdp_register(int svclass, int channel)
+{
+#ifdef HAVE_BLUETOOTH
+	return btkit_register_obex(svclass, channel);
+#else
+	return 0;
+#endif /* HAVE_BLUETOOTH */
+}
+
+
+int obexftp_sdp_unregister(int svclass)
+{
+#ifdef HAVE_BLUETOOTH
+	return btkit_unregister_service(svclass);
+#else
+	return 0;
+#endif /* HAVE_BLUETOOTH */
+}
+
+
+/**
+	Device discovery wrapper for a named transport.
+
+	\param transport a transport from the OBEX_TRANS_x enum.
+
+	\return the discovery results as array of strings.
+
+	\note USB and BT only for now.
+ */
 char **obexftp_discover(int transport)
 {
 	switch (transport)	{
