@@ -1,22 +1,22 @@
-/*
- *  obexftp/client.h: ObexFTP client library
- *
- *  Copyright (c) 2002 Christian W. Zuckschwerdt <zany@triq.net>
- *
- *  This program is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by the Free
- *  Software Foundation; either version 2 of the License, or (at your option)
- *  any later version.
- *
- *  This program is distributed in the hope that it will be useful, but
- *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- *  for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *     
+/**
+	\file obexftp/client.h
+	ObexFTP client API.
+	ObexFTP library - language bindings for OBEX file transfer.
+
+	Copyright (c) 2002-2007 Christian W. Zuckschwerdt <zany@triq.net>
+
+	ObexFTP is free software; you can redistribute it and/or modify
+	it under the terms of the GNU Lesser General Public License as
+	published by the Free Software Foundation; either version 2 of
+	the License, or (at your option) any later version.
+
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU Lesser General Public License for more details.
+
+	You should have received a copy of the GNU Lesser General Public
+	License along with ObexFTP. If not, see <http://www.gnu.org/>.
  */
 
 #ifndef OBEXFTP_CLIENT_H
@@ -56,20 +56,14 @@ extern "C" {
 #define DEFAULT_CACHE_TIMEOUT 180	/* 3 minutes */
 #define DEFAULT_CACHE_MAXSIZE 10240	/* 10k */
 
-/* bt svclass */
-#define OBEX_SYNC_SERVICE	0x1104
-#define OBEX_PUSH_SERVICE	0x1105
-#define OBEX_FTP_SERVICE	0x1106
-
 /* types */
 
-typedef struct stat_entry stat_entry_t;
-struct stat_entry {
+typedef struct {
 	char name[256];
 	mode_t mode;
 	int size;
 	time_t mtime;
-};
+} stat_entry_t;
 
 typedef struct cache_object cache_object_t;
 struct cache_object
@@ -83,8 +77,7 @@ struct cache_object
 	stat_entry_t *stats;	/* only if its a parsed directory */
 };
 
-typedef struct obexftp_client
-{
+typedef struct {
 	/* state */
 	obex_t *obexhandle;
 	uint32_t connection_id; /* set to 0xffffffff if unused */
@@ -125,14 +118,15 @@ typedef struct obexftp_client
 
 void obexftp_close(/*@only@*/ /*@out@*/ /*@null@*/ obexftp_client_t *cli);
 
-char **obexftp_discover(int transport);
-char **obexftp_discover_bt(void); /* this is a quick hack */
-int obexftp_scan_bt(char *addr, int svclass);
-
 int obexftp_connect_uuid(obexftp_client_t *cli,
 				/*@null@*/ const char *device, /* for INET, BLUETOOTH */
 				int port, /* INET(?), BLUETOOTH, USB*/
 				/*@null@*/ const uint8_t uuid[], uint32_t uuid_len);
+int obexftp_connect_src(obexftp_client_t *cli,
+				/*@null@*/ const char *src, /* HCI no. or address */
+				/*@null@*/ const char *device, /* for INET, BLUETOOTH */
+				int port, /* INET(?), BLUETOOTH, USB*/
+				/*@null@*/ const uint8_t uuid[], uint32_t uuid_len);
 
 //int obexftp_connect_service(obexftp_client_t *cli,
 //				/*@null@*/ const char *service,
